/*
 * Universelle
 * LCD-Display Steuerung
 *
 * Copyright (c) 2010, Michael Hohl
 *
 */
 
#include <avr/io.h>
#include "lcd.h"
#include <util/delay.h>
 
/*
 * Erzeugt einen Enable-Puls
 *
 */
static void lcd_enable( void )
{
    LCD_PORT |= (1<<LCD_EN);     // Enable auf 1 setzen
    _delay_us( LCD_ENABLE_US );  // kurze Pause
    LCD_PORT &= ~(1<<LCD_EN);    // Enable auf 0 setzen
}
 
/*
 * Sendet eine 4-bit Ausgabeoperation an das LCD
 *
 */
static void lcd_out( uint8_t data )
{
    data &= 0xF0;                       // obere 4 Bit maskieren
 
    LCD_PORT &= ~(0xF0>>(4-LCD_DB));    // Maske lschen
    LCD_PORT |= (data>>(4-LCD_DB));     // Bits setzen
    lcd_enable();
}
 
/*
 * Initialisiert den LCD-Kontroller.
 *
 */
void lcd_init( void )
{
	// verwendete Pins auf Ausgang schalten
    uint8_t pins = (0x0F << LCD_DB) |           // 4 Datenleitungen
                   (1<<LCD_RS) |                // R/S Leitung
                   (1<<LCD_EN);                 // Enable Leitung
    LCD_DDR |= pins;
 
	LCD_PORT &= ~(1<<LCD_RS);      // RS auf 0, "Befehl"

	// Function set
	LCD_PORT |= (0x03 << LCD_DB);
	lcd_enable();
	LCD_PORT |= (0x03 << LCD_DB);
	lcd_enable();
	LCD_PORT |= (0x03 << LCD_DB);
	lcd_enable();
	_delay_ms(1);

	// Display on/off control 
	LCD_PORT = (0x00 << LCD_DB);
	lcd_enable();
	LCD_PORT |= (0x0F << LCD_DB);
	lcd_enable();
	_delay_ms(1);

	// entry mode
	LCD_PORT = (0x00 << LCD_DB);
	lcd_enable();
	LCD_PORT |= (0x01 << LCD_DB);
	lcd_enable();
	_delay_ms(1);

	// entry mode
	LCD_PORT = (0x00 << LCD_DB);
	lcd_enable();
	LCD_PORT |= (0x07 << LCD_DB);
	lcd_enable();
	_delay_ms(1); 

	
	lcd_clear();
	lcd_home();
}
  
/*
 * Sendet Daten an den LCD-Kontroller.
 *
 */
void lcd_data( uint8_t data )
{
    LCD_PORT |= (1<<LCD_RS);    // RS auf 1 setzen
 
    lcd_out( data );            // zuerst die oberen, 
    lcd_out( data<<4 );         // dann die unteren 4 Bit senden
 
    _delay_us( LCD_WRITEDATA_US );
}
 
/*
 * Schreibt einen String auf das Display.
 *
 */
void lcd_command( uint8_t data )
{
    LCD_PORT &= ~(1<<LCD_RS);    // RS auf 0 setzen
 
    lcd_out( data );             // zuerst die oberen, 
    lcd_out( data<<4);           // dann die unteren 4 Bit senden
 
    _delay_us(LCD_COMMAND_US );
}
 
/*
 * Lscht den Bildschirminhalt des LCDs
 * und setzt den Cursor zurck auf HOME.
 *
 */
void lcd_clear( void )
{
    lcd_command( LCD_CLEAR_DISPLAY );
    _delay_ms( LCD_CLEAR_DISPLAY_MS );
}
 
/*
 * Setzt den Cursor auf HOME
 * (HOME = Ausgangsposition)
 *
 */
void lcd_home( void )
{
    lcd_command( LCD_CURSOR_HOME );
    _delay_ms( LCD_CURSOR_HOME_MS );
}
 
/*
 * Setzt den Cursor an eine beliebige Position.
 *
 */
void lcd_setcursor( uint8_t x, uint8_t y )
{
    uint8_t data;
 
    switch (y)
    {
        case 1:    // 1. Zeile
            data = LCD_SET_DDADR + LCD_DDADR_LINE1 + x;
            break;
 
        case 2:    // 2. Zeile
            data = LCD_SET_DDADR + LCD_DDADR_LINE2 + x;
            break;
 
        case 3:    // 3. Zeile
            data = LCD_SET_DDADR + LCD_DDADR_LINE3 + x;
            break;
 
        case 4:    // 4. Zeile
            data = LCD_SET_DDADR + LCD_DDADR_LINE4 + x;
            break;
 
        default:
            return;                                   // fr den Fall einer falschen Zeile
    }
 
    lcd_command( data );
}
 
/*
 * Schreibt einen String auf das LCD
 *
 */
void lcd_string( const char *data )
{
    while( *data != '\0' )
        lcd_data( *data++ );
}
 
/*
 * Definition eines benutzerdefinierten Sonderzeichens.
 * data muss auf ein Array[5] mit den Spaltencodes des zu definierenden Zeichens
 * zeigen
 *
 */
void lcd_generatechar( uint8_t code, const uint8_t *data )
{
    // Startposition des Zeichens einstellen
    lcd_command( LCD_SET_CGADR | (code<<3) );
 
    // Bitmuster bertragen
    for ( uint8_t i=0; i<8; i++ )
    {
        lcd_data( data[i] );
    }
}
